require 'spec_helper'

describe AppConfig::Storage::YAML do

  it 'should have some values' do
    config_for_yaml
    AppConfig.api_key.should_not be_nil
  end

  it 'should raise file not found' do
    lambda do
      config_for_yaml(yaml: 'not/a/real/file.yml')
    end.should raise_error(Errno::ENOENT)
  end

  it 'saves the new value in memory' do
    config_for_yaml
    AppConfig.new_key = 'new value'
    AppConfig.new_key.should == 'new value'
  end

  it "requires the use of 'Dir.home'" do
    unless Dir.respond_to?(:home)
      fail "Requires 'Dir.home' which is available in Ruby 1.9"
    end
  end

  it "uses the defaults when 'true' is passed" do
    AppConfig.reset!

    # Hack to use spec config as the 'default'
    AppConfig::Storage::YAML::DEFAULT_PATH = fixture('app_config.yml')

    AppConfig.setup!(yaml: true)
    AppConfig.api_key.should_not be_nil
  end
end
